@@ -55,12 +55,13 @@ def mpi_enabled():
 
 
 class BuildConfig:
-    def __init__(self, hdf5_includedirs, hdf5_libdirs, hdf5_define_macros, hdf5_version, mpi):
+    def __init__(self, hdf5_includedirs, hdf5_libdirs, hdf5_define_macros, hdf5_version, mpi, ros3):
         self.hdf5_includedirs = hdf5_includedirs
         self.hdf5_libdirs = hdf5_libdirs
         self.hdf5_define_macros = hdf5_define_macros
         self.hdf5_version = hdf5_version
         self.mpi = mpi
+        self.ros3 = ros3
 
     @classmethod
     def from_env(cls):
@@ -70,10 +71,15 @@ class BuildConfig:
         h5_version_s = os.environ.get('HDF5_VERSION')
         if h5_version_s:
             h5_version = validate_version(h5_version_s)
+            h5_wrapper = HDF5LibWrapper(h5_lib)
         else:
-            h5_version = autodetect_version(h5_lib)
+            h5_wrapper = HDF5LibWrapper(h5_lib)
+            h5_version = h5_wrapper.autodetect_version()
+            if mpi and not h5_wrapper.has_mpi_support():
+                raise RuntimeError("MPI support not detected")
+        ros3 = h5_wrapper.has_ros3_support()
 
-        return cls(h5_inc, h5_lib, h5_macros, h5_version, mpi)
+        return cls(h5_inc, h5_lib, h5_macros, h5_version, mpi, ros3)
 
     @staticmethod
     def _find_hdf5_compiler_settings(mpi=False):
@@ -147,6 +153,7 @@ class BuildConfig:
             'hdf5_define_macros': self.hdf5_define_macros,
             'hdf5_version': list(self.hdf5_version),  # list() to match the JSON
             'mpi': self.mpi,
+            'ros3': self.ros3,
         }
 
     def changed(self):
@@ -168,61 +175,111 @@ class BuildConfig:
         print("HDF5 library dirs:", fmt_dirs(self.hdf5_libdirs))
         print("     HDF5 Version:", repr(self.hdf5_version))
         print("      MPI Enabled:", self.mpi)
+        print(" ROS3 VFD Enabled:", self.ros3)
         print(" Rebuild Required:", self.changed())
         print('')
         print('*' * 80)
 
 
-def autodetect_version(libdirs):
-    """
-    Detect the current version of HDF5, and return X.Y.Z version string.
+class HDF5LibWrapper:
 
-    Intended for Unix-ish platforms (Linux, OS X, BSD).
-    Does not support Windows. Raises an exception if anything goes wrong.
+    def __init__(self, libdirs):
+        self._load_hdf5_lib(libdirs)
 
-    config: configuration for the build (configure command)
-    """
-    import re
-    import ctypes
-    from ctypes import byref
+    def _load_hdf5_lib(self, libdirs):
+        """
+        Detect and load the HDF5 library.
 
-    if sys.platform.startswith('darwin'):
-        default_path = 'libhdf5.dylib'
-        regexp = re.compile(r'^libhdf5.dylib')
-    elif sys.platform.startswith('win') or \
-        sys.platform.startswith('cygwin'):
-        default_path = 'hdf5.dll'
-        regexp = re.compile(r'^hdf5.dll')
-    else:
-        default_path = 'libhdf5.so'
-        regexp = re.compile(r'^libhdf5.so')
+        Raises an exception if anything goes wrong.
+
+        libdirs: the library paths to search for the library
+        """
+        import re
+        import ctypes
+
+        # extra keyword args to pass to LoadLibrary
+        load_kw = {}
+        if sys.platform.startswith('darwin'):
+            default_path = 'libhdf5.dylib'
+            regexp = re.compile(r'^libhdf5.dylib')
+        elif sys.platform.startswith('win') or \
+            sys.platform.startswith('cygwin'):
+            default_path = 'hdf5.dll'
+            regexp = re.compile(r'^hdf5.dll')
+            if sys.version_info >= (3, 8):
+                # To overcome "difficulty" loading the library on windows
+                # https://bugs.python.org/issue42114
+                load_kw['winmode'] = 0
+        else:
+            default_path = 'libhdf5.so'
+            regexp = re.compile(r'^libhdf5.so')
+
+        path = None
+        for d in libdirs:
+            try:
+                candidates = [x for x in os.listdir(d) if regexp.match(x)]
+            except Exception:
+                continue   # Skip invalid entries
+
+            if len(candidates) != 0:
+                candidates.sort(key=lambda x: len(x))   # Prefer libfoo.so to libfoo.so.X.Y.Z
+                path = op.abspath(op.join(d, candidates[0]))
+                break
+
+        if path is None:
+            path = default_path
+
+        print("Loading library to get build settings and version:", path)
+
+        self._lib_path = path
+
+        if op.isabs(path) and not op.exists(path):
+            raise FileNotFoundError(f"{path} is missing")
 
-    path = None
-    for d in libdirs:
         try:
-            candidates = [x for x in os.listdir(d) if regexp.match(x)]
+            lib = ctypes.CDLL(path, **load_kw)
         except Exception:
-            continue   # Skip invalid entries
+            print("error: Unable to load dependency HDF5, make sure HDF5 is installed properly")
+            raise
 
-        if len(candidates) != 0:
-            candidates.sort(key=lambda x: len(x))   # Prefer libfoo.so to libfoo.so.X.Y.Z
-            path = op.abspath(op.join(d, candidates[0]))
-            break
+        self._lib = lib
 
-    if path is None:
-        path = default_path
+    def autodetect_version(self):
+        """
+        Detect the current version of HDF5, and return X.Y.Z version string.
 
-    major = ctypes.c_uint()
-    minor = ctypes.c_uint()
-    release = ctypes.c_uint()
+        Raises an exception if anything goes wrong.
+        """
+        import ctypes
+        from ctypes import byref
 
-    print("Loading library to get version:", path)
+        major = ctypes.c_uint()
+        minor = ctypes.c_uint()
+        release = ctypes.c_uint()
 
-    try:
-        lib = ctypes.cdll.LoadLibrary(path)
-        lib.H5get_libversion(byref(major), byref(minor), byref(release))
-    except Exception:
-        print("error: Unable to load dependency HDF5, make sure HDF5 is installed properly")
-        raise
+        try:
+            self._lib.H5get_libversion(byref(major), byref(minor), byref(release))
+        except Exception:
+            print("error: Unable to find HDF5 version")
+            raise
 
-    return int(major.value), int(minor.value), int(release.value)
+        return int(major.value), int(minor.value), int(release.value)
+
+    def load_function(self, func_name):
+        try:
+            return getattr(self._lib, func_name)
+        except AttributeError:
+            # No such function
+            return None
+
+    def has_functions(self, *func_names):
+        for func_name in func_names:
+            if self.load_function(func_name) is None:
+                return False
+        return True
+
+    def has_mpi_support(self):
+        return self.has_functions("H5Pget_fapl_mpio", "H5Pset_fapl_mpio")
+
+    def has_ros3_support(self):
+        return self.has_functions("H5Pget_fapl_ros3", "H5Pset_fapl_ros3")
